(* fsm2.sml
 *
 * Non-deterministic and deterministic finite-state machines.
 *
 * This new implementation uses a compressed representation for the state
 * table.  NFA->DFA should also be a lot faster because we now deal strictly
 * with equivalence classes.
 *)

structure Dfa2 :> DFA = 
struct

   structure R   = RegExpSyntax
   structure A   = Array
   structure DA  = DenseAutomataTable
   structure Eqv = EquivCharClass

   val error = 0

   datatype dfa = 
       DFA of
       {base :   int A.array,  (* state -> offset *)
        equiv:   int A.array,  (* char -> equiv class *)
        next :   int option A.array,  (* offset -> state *)
        check :  int A.array,  (* offset -> state *)
        default: int A.array,  (* state -> state *)
        accept:  int option A.array   (* state -> rule *)
       }

   exception SyntaxNotHandled = R.CannotCompile

   fun buildPattern patterns =
   let 
       (* Compute the equivalence classes *)
       val {mapping=equiv, numclasses, ...} = Eqv.equivClasses patterns
       fun eqv c = A.sub(equiv,ord c)

       (* Build the nfa *)

   in  raise R.CannotCompile
   end
   
   fun build regexp = buildPattern [regexp]

   fun accepting (DFA{accept, ...}) state = A.sub(accept, state)

   fun move (DFA{base, equiv, next, check, default, ...}) (state, ch) =
   let val eqv = A.sub(equiv, ord ch)
       fun delta(s, c) =
       let val offset = A.sub(base, s) + c
       in  if A.sub(check, offset) = s 
           then A.sub(next, offset)
           else let val s' = A.sub(default, s)
                in  if s' = error then NONE else delta(s', c)
                end
       end
   in  delta(state, eqv)
   end
 
   fun canStart _ = raise R.CannotCompile

end
