(*
 * This module is responsible for partitioning the character set into
 * equivalence classes
 *)
structure EquivClassTable :> EQUIV_CLASS_TABLE =
struct
   structure A = Array
   structure H = HashTable
   type symbol = int

   datatype equivclass = 
      EQV of {stamp   : word ref,
              minChar : symbol,
              maxChar : symbol,
              classes : word list A.array
             }

   fun new{min,max} = 
   let val n = max+1 
   in  EQV{stamp=ref 0w0, classes=A.array(n,[]), minChar=min, maxChar=max}
   end
       
   fun set (EQV{stamp, classes, ...}) s = 
   let val st = !stamp
       fun mark [] = ()
         | mark(c::cs) = (A.update(classes, c, st::A.sub(classes, c)); mark cs)
   in  stamp := st + 0w1; 
       mark s
   end

   fun single (EQV{stamp, classes, ...}) c = 
   let val st = !stamp
   in  stamp := st + 0w1;
       A.update(classes, c, [st])
   end

   fun classes(EQV{classes, minChar, maxChar, ...}) = 
   let exception Classes
       fun h([],w) = w
         | h(x::xs,w) = h(xs, x+Word.<<(w,0w2))
       fun hash xs = h(xs, 0w0)
       fun eq(x,y:word list) = x=y
       val table  = H.mkTable (hash,eq) (16,Classes)
       val find   = H.find table
       val insert = H.insert table
       val numclasses = ref 0
   in  A.appi (fn (c,marker) => 
               case find marker of
                NONE => (insert(marker,(!numclasses,[c])); 
                         numclasses := !numclasses+1)
              | SOME(k,cs) => insert(marker,(k,c::cs)))
                (classes, minChar, NONE);
       {numclasses = !numclasses,
        partitions = H.listItems table
       }
   end
 
   fun mapping (eqv as EQV{maxChar, ...}) =
   let val {numclasses, partitions, ...} = classes eqv
       val m = A.array(maxChar+1,~1)
   in  app (fn (k,cs) => app (fn c => A.update(m,c,k)) cs) partitions;
       {numclasses=numclasses, mapping=m}
   end
       

end
