(*
 * A generic lexer generator.  
 *
 * Currently it supports the following features:
 *
 * 1. The meta characters ^ and $
 * 2. lookahead
 * 3. contexts
 * 4. wider characters than 8 bit (for future unicode support)
 *
 *)
signature LEXER_GENERATOR =
sig
   structure R     : GENERALIZED_REGEXP_SYNTAX
   structure Lexer : LEXER_TABLES

     (* a user defined start states; these most be numbered from 1 *)
   type context = int 

   datatype rule = (* a matching rule *)
      RULE of
      { contexts  : context list,     (* user defined contexts *)
        regexp    : R.syntax,         (* the regular expression *)
        lookahead : R.syntax option   (* lookahead (not implemented yet) *)
      }

   (* Create a lexer *)
   val compile : R.syntax list -> Lexer.lexer

   val compile' : 
       { matchtrees  : bool, (* generate tables to keep track of matchtrees?*)
         numContexts : int   (* number of special contexts *)
       } -> rule list -> Lexer.lexer
end
