(* syntax.sml
 *
 * COPYRIGHT (c) 1995 AT&T Bell Laboratories.
 *
 * This is the abstract syntax tree used to represent regular expressions.
 * It serves as the glue between different front-ends (implementing
 * different RE specification languages), and different back-ends (implementing
 * different compilation/searching algorithms).
 *)
(*GC44 CHANGES
  Converted "functor GENERALIZED_REGEXP_SYNTAX" to "structure REGEXP_SYNTAX"
*)

structure RegExpSyntax : REGEXP_SYNTAX =
  struct
 
      exception CannotParse
      exception CannotCompile
  
      structure Char = Char

       (* Use something faster than ListSet -- Allen *)
      structure CharSet = 
         SplaySetFn (struct type ord_key = Char.char 
                            val compare = Char.compare 
                     end)

      datatype syntax
	= Group of syntax
	| Alt of syntax list
	| Concat of syntax list
	| Interval of (syntax * int * int option)
	| Option of syntax	(* == Interval(re, 0, SOME 1) *)
	| Star of syntax	(* == Interval(re, 0, NONE) *)
	| Plus of syntax	(* == Interval(re, 1, NONE) *)
	| MatchSet of CharSet.set
	| NonmatchSet of CharSet.set
	| Char of Char.char
	| Begin
	| End

             (* Extensions *)
        | Assign of int * (string -> string) * syntax  
             (* define a reference *)
        | Ref of (string -> string) * int (* back references *)
        | Guard of (string -> bool) * syntax
        | Boundary of {prev: Char.char option, 
                       curr: Char.char option,
                       next: Char.char option} -> bool

      fun addRange (s,minC,maxC) = 
           CharSet.addList (s,List.tabulate (Char.ord(maxC)-Char.ord(minC)+1,
                            fn v => Char.chr (v+Char.ord(minC))))

      val allChars = addRange (CharSet.empty, Char.minChar, Char.maxChar)
	  
  end;
