(*
 * This signature describes modules that perform 
 * compression and encoding of deterministic automata tables.
 *)
signature AUTOMATA_TABLE =
sig

   structure DynArray :
   sig
      type 'a array 
      val sub      : 'a array * int -> 'a
      val update   : 'a array * int * 'a -> unit
      val length   : 'a array -> int
      val array    : int * 'a -> 'a array
      val toList   : 'a array -> 'a list
      val toArray  : 'a array -> 'a Array.array
      val toPacked : int array * int -> string 
      val setBound : 'a array * int -> unit 
   end

   exception AutomataTable of string 
 
   type table
   type state = int
   type symbol = int


       (* Create a new table with only the error state.
        * You must declare the range of input symbols in the character set.
        *)
   val new        : {min:symbol, max:symbol} -> table

       (* The error state *)
   val error      : state

       (* The character range *)
   val charRange  : table -> {min:symbol, max:symbol}

       (* Number of states *)
   val stateCount : table -> int
 
       (* Add a new state *)
   val addState   : table -> state * (symbol * state) list -> unit 

       (* Finish the compression process *)
   val compress   : table -> unit 

       (* Create a function that uses the transition table *)
   val go         : table -> state * symbol -> state 

       (* Extract the internal representation *)
   val getrep     : table -> string -> int DynArray.array
   val getarr     : table -> string -> int Array.array
   val getpacked  : table -> string -> string * int 
   val getwidth   : table -> string -> int 

end
