(*
 * This module is responsible for partitioning the character set into
 * equivalence classes
 *)
signature EQUIV_CLASS_TABLE =
sig
   type equivclass
   type symbol = int

                 (* Create a universe U from {min...max} *)
   val new     : {min:symbol, max:symbol} -> equivclass

                 (* Given c, partition on U - {c} and {c} *)
   val single  : equivclass -> symbol -> unit

                 (* Given S, partition on U - S and S *)
   val set     : equivclass -> symbol list -> unit

                 (* Compute the partitions *)
   val classes : equivclass -> {numclasses : int,
                                partitions : (int * symbol list) list
                               }

                 (* Compute partitions and create lookup table *)
   val mapping  : equivclass -> 
                   {numclasses : int,
                    mapping    : int Array.array (* symbol to equivclass *)
                   }

end
