(* Compute the equivalence classes from a list of regular expressions *)
signature EQUIV_CHAR_CLASS =
sig

   structure R   : GENERALIZED_REGEXP_SYNTAX
   structure Eqv : EQUIV_CLASS_TABLE

   val partition : Eqv.equivclass -> R.syntax -> unit
end

functor EquivCharClassFn(R : GENERALIZED_REGEXP_SYNTAX) : 
   EQUIV_CHAR_CLASS where R = R =
struct

   structure R    = R
   structure Char = R.Char
   structure Eqv  = EquivClassTable

   fun partition eqv =
   let val single = Eqv.single eqv
       val set = Eqv.set eqv
       val toList = R.CharSet.foldr (fn (c,s) => Char.ord c::s) []
       fun walk(R.Group e) = walk e
         | walk(R.Alt es) = app walk es
         | walk(R.Concat es) = app walk es
         | walk(R.Star e) = walk e
         | walk(R.Option e) = walk e
         | walk(R.Plus e) = walk e
         | walk(R.Interval(e,_,_)) = walk e
         | walk(R.Guard(_,e)) = walk e
         | walk(R.Ref _) = ()
         | walk(R.Assign(_,_,e)) = walk e 
         | walk(R.Begin) = ()
         | walk(R.End) = single 10 (* character code is 10 *)
         | walk(R.Boundary _) = ()
         | walk(R.Char c) = single (Char.ord c) 
         | walk(R.MatchSet s) = set (toList s) 
         | walk(R.NonmatchSet s) = set (toList s) 
   in  walk 
   end

end
