(*
 * (Non-deterministic) finite state automata 
 *)
signature FSA =
sig
   type symbol  = int
   type ruleno  = int
   type groupid = int
   type nfa 
   type node
   structure IntSet : ORD_SET where type Key.ord_key = int

   val intsetToString : IntSet.set -> string

   (* methods for constructing an nfa *)
   val nfa        : {min:symbol, max:symbol} -> nfa
   val newNode    : nfa -> node
   val setDelta   : node * symbol * node -> node (* transition node *)
   val setEpsilon : node * node -> node          (* epsilon node *)
   val setOr      : node * node list -> node     (* disjunction *)
   val setAccept  : node * ruleno -> node        (* accept state *)
   val setBeginGroup : node * groupid * node -> node 
   val setEndGroup   : node * groupid * node -> node 
   val setError      : node -> node

   (* compute the epsilon closure *)
   val computeClosure : nfa -> unit

   (* query methods *)
   val closure        : nfa -> node -> IntSet.set
   val move           : nfa -> IntSet.set -> (symbol * IntSet.set) list
   val acceptingAll   : nfa -> IntSet.set -> IntSet.set (* set of rulenos *)
   val accepting      : nfa -> IntSet.set -> ruleno option

   (* Get the begin groups and end groups that can occur in this state 
    * These are indexed by the group ids.
    *)
   val groupings      : nfa -> IntSet.set -> groupid list * groupid list
end
