(*
 * A lazy man's interface to the regexp library.
 *)
signature REGEXP_LIB =
sig
   type regexp = string
   type text   = string

        (* search *)
   val grep  : regexp -> text -> string list option

        (* find all occurrances matching regexp *)
   val findAll : regexp -> text -> string list

        (* find all occurrances of the ith group in the matching regexp *)
   val findAllGroup : regexp -> int -> text -> string list

        (* match and extract the match; return "" otherwise *)
   val extract : regexp -> text -> string

        (* match and extract the ith group; return "" otherwise *)
   val extractGroup : regexp -> int -> text -> string

        (* search only *)
   val matches  : regexp -> text -> bool

        (* search and replace *) 
   val subst : regexp -> (string list -> string) -> text -> text

        (* search and replace all occurrances *) 
   val substAll : regexp -> (string list -> string) -> text -> text

        (* search and replace *) 
   val replace : regexp -> string -> text -> text

        (* search and replace all occurrances *) 
   val replaceAll : regexp -> string -> text -> text

        (* multiway match *)
   val match : text ->
               { cases: (regexp * (string list -> 'a)) list,
                 default: unit -> 'a
               } -> 'a
end
