(******************************************************************************)
theory NJlib
imports Pure
begin
(******************************************************************************)

(*----------------------------------------------------------------------------*)
(*\<bar>-- Documentation ---------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)
(*
mllib\nj\RegExp.txt and Util sig files

http://www.smlnj.org/doc/smlnj-lib/Manual/toc.html
http://www.smlnj.org/doc/smlnj-lib/Manual/regexp-lib-part.html
http://www.smlnj.org/doc/SMLofNJ/pages/index-all.html
*)

(*----------------------------------------------------------------------------*)
(*\<bar>-- Dependencies (Mostly for RegExp) --------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

ML_file "nj/Util/lib-base-sig.sml"
ML_file "nj/Util/lib-base.sml" (*
  \<bullet> String.concat.*)
ML_file "nj/Util/ord-key-sig.sml"
ML_file "nj/Util/ord-set-sig.sml"
ML_file "nj/Util/list-set-fn.sml"
ML_file "nj/Util/ord-map-sig.sml"
ML_file "nj/Util/list-map-fn.sml"
  
(*----------------------------------------------------------------------------*)
(*\<bar>-- REGULAR EXPRESSIONS ---------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

(*\<bar>-----------------\<bar>*)
(*\<bar> RegExp/Frontend \<bar>*)
(*\<bar>-----------------\<bar>*)

ML_file "nj/RegExp/FrontEnd/syntax-sig.sml"
ML_file "nj/RegExp/FrontEnd/syntax.sml" (*
  \<bullet> Char.ord, Char.chr.*)
ML_file "nj/RegExp/FrontEnd/parser-sig.sml"
ML_file "nj/RegExp/FrontEnd/awk-syntax.sml" (*
  \<bullet> String.explode, Option.valOf.*)

(*\<bar>-----------------------------------------------\<bar>*)  
(*\<bar> RegExp/Glue and RegExp/Backend/engine-sig.sml \<bar>*)
(*\<bar>-----------------------------------------------\<bar>*) 

ML_file "nj/RegExp/Glue/match-tree.sml"
ML_file "nj/RegExp/Glue/regexp-sig.sml"
ML_file "nj/RegExp/BackEnd/engine-sig.sml"

ML_file "nj/RegExp/Glue/regexp-fn.sml"


(*\<bar>----------------\<bar>*)
(*\<bar> RegExp/Backend \<bar>*)
(*\<bar>----------------\<bar>*)

ML_file "nj/RegExp/BackEnd/bt-engine.sml"
ML_file "nj/RegExp/BackEnd/fsm.sml" (*
  \<bullet> List.foldr, List.foldl, Unsynchronized.ref, Option.valOf, Char.chr, 
    Char.ord.*)
ML_file "nj/RegExp/BackEnd/dfa-engine.sml"
ML_file "nj/RegExp/BackEnd/thompson-engine.sml" (*
  \<bullet> Unsynchronized.ref, String.concat, print.*)

(*\<bar>-------------------------\<bar>*)
(*\<bar> RegExpTests/test-fn.sml \<bar>*)
(*\<bar>-------------------------\<bar>*)

(*GC44 CHANGES:
  String.concat to String.concat
  writeln to writeln
*)
ML{*
functor TestFn (
  val engineName : string
  structure RE : REGEXP
) = struct

  structure M = MatchTree

  fun getc s i = if (i < String.size s) then SOME(String.sub(s, i), i+1) 
                 else NONE

  fun test (name, re, data) = let
  val _ = writeln(String.concat["  ", name, ": "])
  val re = RE.compileString re handle ex => (writeln "compile failed\n"; raise ex)
  in
    case ((RE.find re (getc data) 0) handle ex => NONE)
     of NONE => writeln "match failed\n"
      | SOME(M.Match({pos, len}, _), _) =>
    writeln(String.concat[
        "match at ", Int.toString pos, " = \"",
        String.toString(String.substring(data, pos, len)), "\"\n"
      ])
    (* end case *)
  end
    handle _ => ()

  fun doTests () = (
  writeln(String.concat["  testing ", engineName, "\n"]);
  test ("01", "[0-9]+", "abc123xyz");
  test ("02", "^[0-9]+", "abc123def\n987xyz");
  test ("03", "[0-9]+$", "abc123def\n987xyz456");
  test ("04", "[0-9]+$", "987xyz456\nabc123");
  test ("05", "^$", "");
  test ("06", ".", "a");
  test ("07", "^foo$", "foo");
  test ("08", "^...$", "foo");
  test ("09", "^.*$", "foo");
  test ("10", "^.*foo@bar\\.com$", "foo@bar.com");
  test ("11", "(abc)","abc");
  test ("12", "\\(abc\\)","(abc)");
  test ("13", "(abc){2,4}$", "abcabc");
  test ("14", "(abc){2,4}$", "abcabcabc");
  test ("15", "(abc){2,4}$", "abcabcabcabc");
  writeln "** tests done\n")

end
*}

(*\<bar>--------------------------\<bar>*)
(*\<bar> RegExp/Tests/engines.sml \<bar>*)
(*\<bar>--------------------------\<bar>*)

ML{*
structure ThompsonRE = TestFn (
    val engineName = "Thompson's engine"
    structure RE = RegExpFn(
      structure P = AwkSyntax
      structure E = ThompsonEngine))

structure DfaRE = TestFn (
    val engineName = "DFA engine"
    structure RE = RegExpFn(
      structure P = AwkSyntax
      structure E = DfaEngine))

structure BackTrackRE = TestFn (
    val engineName = "Back-tracking engine"
    structure RE = RegExpFn(
      structure P = AwkSyntax
      structure E = BackTrackEngine))
*}

(*\<bar>---------------------\<bar>*)
(*\<bar> RegExp/Tests/run.sh \<bar>*)
(*\<bar>---------------------\<bar>*)

ML{*ThompsonRE.doTests();*}
ML{*DfaRE.doTests();*}
ML{*BackTrackRE.doTests();*}

(*----------------------------------------------------------------------------*)
(*\<bar>-- ITERATE ---------------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

ML_file "nj/Util/iterate-sig.sml"
ML_file "nj/Util/iterate.sml"

(*----------------------------------------------------------------------------*)
(*\<bar>-- RANDOM NUMBERS --------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

(*\<bar>--------\<bar>*)
(*\<bar> Random \<bar>*)
(*\<bar>--------\<bar>*)

ML_file "nj/Util/random-sig.sml"
ML_file "nj/Util/random.sml" (*
  \<bullet> Word32, Unsynchronized.ref.*)
ML{*
  val IzSml_Random_seed      = Random.rand(16807,2147483647);
  val IzSml_Random_randRange = Random.randRange 
                             (0,0xffffffffffffffff) IzSml_Random_seed;
*}

(*\<bar>------\<bar>*)
(*\<bar> Rand \<bar>*)
(*\<bar>------\<bar>*)

ML_file "nj/Util/rand-sig.sml" (*
  \<bullet> Word32.*)
ML_file "nj/Util/rand.sml" (*
  \<bullet> Word32, Unsynchronized.ref.*)
ML{*
                            Rand.randMin;
                            Rand.randMax;
  val IzSml_Rand_random1  = Rand.random (Word32.fromInt (1404131431));
  val IzSml_Rand_random2  = Rand.random  IzSml_Rand_random1;
  
  val IzSml_Rand_mkRandom = Rand.mkRandom (Word32.fromInt (1404131431));
  Library.radixstring(26,"a", Word32.toInt (IzSml_Rand_mkRandom()));
  Library.radixstring(26,"a", Word32.toInt (IzSml_Rand_mkRandom()));
  Library.radixstring(26,"a", Word32.toInt (IzSml_Rand_mkRandom()));
*}

(*----------------------------------------------------------------------------*)
(*\<bar>-- HASH TABLES -----------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

ML_file "nj/Util/hash-key-sig.sml"
ML_file "nj/Util/hash-set-fn.sml" (*
  \<bullet> Unsynchronized.ref.*)
ML_file "nj/Util/hash-string.sml" (*
  \<bullet> Unsafe.CharVector.*) 
ML_file "nj/Util/hash-table-rep.sml" (*
  \<bullet> Unsynchronized.ref.*)
ML_file "nj/Util/mono-hash-table-sig.sml"  
ML_file "nj/Util/mono-hash2-table-sig.sml"  
ML_file "nj/Util/hash-table-fn.sml" (*
  \<bullet> Unsynchronized.ref.*)
ML_file "nj/Util/hash2-table-fn.sml" (*
  \<bullet> Unsynchronized.ref.*) 
ML_file "nj/Util/hash-table-sig.sml"
ML_file "nj/Util/hash-table.sml" (*
  \<bullet> Unsynchronized.ref.*)
ML_file "nj/Util/int-hash-table.sml" (*
  \<bullet> Unsynchronized.ref.*)  
ML_file "nj/Util/word-hash-table.sml" (*
  \<bullet> Unsynchronized.ref.*)  

(*----------------------------------------------------------------------------*)
(*\<bar>-- ATOMS -----------------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

ML_file "nj/Util/atom-sig.sml"
ML_file "nj/Util/atom.sml" (*
  \<bullet> Unsynchronized.ref.*)  

(*----------------------------------------------------------------------------*)
(*\<bar>-- ARRAYS ----------------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

ML_file "nj/Util/bit-array-sig.sml"
ML_file "nj/Util/bit-array.sml" (*
  \<bullet> String.explode, List.foldl.*)  
ML_file "nj/Util/array2-sig.sml"
ML_file "nj/Util/array2.sml"
ML_file "nj/Util/mono-array-sort-sig.sml"
ML_file "nj/Util/mono-array-fn.sml"
ML_file "nj/Util/mono-dynamic-array-sig.sml"  
ML_file "nj/Util/dynamic-array-fn.sml" (*
  \<bullet> Unsynchronized.ref.*)  
ML_file "nj/Util/dynamic-array-sig.sml"
ML_file "nj/Util/dynamic-array.sml" (*
  \<bullet> Unsynchronized.ref.*)
ML_file "nj/Util/array-sort-sig.sml"
ML_file "nj/Util/array-qsort.sml" (*
  \<bullet> Unsafe.Array.*) 
ML_file "nj/Util/array-qsort-fn.sml"

(*----------------------------------------------------------------------------*)
(*\<bar>-- FIFO ------------------------------------------------------------------\<bar>*)
(*----------------------------------------------------------------------------*)

ML_file "nj/Util/fifo-sig.sml"
ML_file "nj/Util/fifo.sml"

(******************************************************************************)
end
(******************************************************************************)






